@@ -5,11 +5,10 @@ from __future__ import annotations
 import asyncio
 from contextvars import ContextVar
 from datetime import datetime, timedelta
-from enum import Enum
 import logging
 import random
 from ssl import SSLContext, SSLError
-from typing import TYPE_CHECKING, cast
+from typing import TYPE_CHECKING
 
 import aiohttp
 import async_timeout
@@ -18,9 +17,14 @@ from snitun.exceptions import SniTunConnectionError
 from snitun.utils.aes import generate_aes_keyset
 from snitun.utils.aiohttp_client import SniTunClientAioHttp
 
-from . import cloud_api, const, utils
+from . import const, utils
 from .acme import AcmeClientError, AcmeHandler, AcmeJWSVerificationError
-from .const import SubscriptionReconnectionReason
+from .const import (
+    DISPATCH_CERTIFICATE_STATUS,
+    CertificateStatus,
+    SubscriptionReconnectionReason,
+)
+from .instance_api import InstanceApiError
 
 if TYPE_CHECKING:
     from . import Cloud, _ClientT
@@ -79,16 +83,6 @@ class Certificate:
     alternative_names = attr.ib(type=list[str] | None)
 
 
-class CertificateStatus(str, Enum):
-    """Representation of the remote UI status."""
-
-    ERROR = "error"
-    GENERATING = "generating"
-    LOADED = "loaded"
-    LOADING = "loading"
-    READY = "ready"
-
-
 class RemoteUI:
     """Class to help manage remote connections."""
 
@@ -139,6 +133,18 @@ class RemoteUI:
         """Return the certificate status."""
         return self._certificate_status
 
+    def _update_certificate_status(
+        self,
+        status: CertificateStatus,
+    ) -> None:
+        """Update certificate status and notify via dispatcher."""
+        if self._certificate_status == status:
+            return
+
+        self._certificate_status = status
+        _LOGGER.debug("Certificate status: %s", status)
+        self.cloud.client.dispatcher_message(DISPATCH_CERTIFICATE_STATUS, status)
+
     @property
     def instance_domain(self) -> str | None:
         """Return instance domain."""
@@ -196,30 +202,37 @@ class RemoteUI:
         await asyncio.sleep(5)
         await self.load_backend()
 
+    def _generate_acme_handler(self, *, domains: list[str], email: str) -> AcmeHandler:
+        """Generate a new ACME handler."""
+        return AcmeHandler(
+            self.cloud,
+            domains,
+            email,
+            status_callback=self._update_certificate_status,
+        )
+
     async def _recreate_acme(self, domains: list[str], email: str) -> None:
         """Recreate the acme client."""
-        if self._acme and self._acme.certificate_available:
+        if self._acme:
             await self._acme.reset_acme()
-        self._acme = AcmeHandler(self.cloud, domains, email)
+        self._acme = self._generate_acme_handler(domains=domains, email=email)
 
     async def load_backend(self) -> bool:
         """Load backend details."""
         try:
             async with async_timeout.timeout(30):
-                resp = await cloud_api.async_remote_register(self.cloud)
-            resp.raise_for_status()
-        except (TimeoutError, aiohttp.ClientError) as err:
+                data = await self.cloud.instance.register()
+        except (TimeoutError, InstanceApiError) as err:
             msg = "Can't update remote details from Home Assistant cloud"
-            if isinstance(err, aiohttp.ClientResponseError):
-                msg += f" ({err.status})"  # pylint: disable=no-member
-            elif isinstance(err, asyncio.TimeoutError):
+            if isinstance(err, TimeoutError):
                 msg += " (timeout)"
+            else:
+                msg += f" ({err})"
             _LOGGER.error(msg)
             return False
-        data = await resp.json()
 
         # Extract data
-        _LOGGER.debug("Retrieve instance data: %s", data)
+        _LOGGER.debug("Retrieved instance data: %s", data)
 
         instance_domain = data["domain"]
         email = data["email"]
@@ -228,15 +241,15 @@ class RemoteUI:
         # Cache data
         self._instance_domain = instance_domain
         self._snitun_server = server
-        self._alias = cast("list[str]", data.get("alias", []))
+        self._alias = data.get("alias", [])
 
         domains: list[str] = [instance_domain, *self._alias]
 
         # Set instance details for certificate
-        self._acme = AcmeHandler(self.cloud, domains, email)
+        self._acme = self._generate_acme_handler(domains=domains, email=email)
 
         # Load exists certificate
-        self._certificate_status = CertificateStatus.LOADING
+        self._update_certificate_status(CertificateStatus.LOADING)
         await self._acme.load_certificate()
 
         # Domain changed / revoke CA
@@ -249,8 +262,7 @@ class RemoteUI:
         ):
             for alias in self.alias or []:
                 if not await self._custom_domain_dns_configuration_is_valid(
-                    instance_domain,
-                    alias,
+                    instance_domain, alias
                 ):
                     domains.remove(alias)
 
@@ -268,7 +280,7 @@ class RemoteUI:
 
         if should_create_cert:
             try:
-                self._certificate_status = CertificateStatus.GENERATING
+                self._update_certificate_status(CertificateStatus.INITIAL_GENERATING)
                 await self._acme.issue_certificate()
             except (AcmeJWSVerificationError, AcmeClientError) as err:
                 if isinstance(err, AcmeJWSVerificationError):
@@ -282,7 +294,7 @@ class RemoteUI:
                     "Home Assistant Cloud",
                     const.MESSAGE_REMOTE_SETUP,
                 )
-                self._certificate_status = CertificateStatus.ERROR
+                self._update_certificate_status(CertificateStatus.INITIAL_CERT_ERROR)
                 return False
 
             self.cloud.client.user_message(
@@ -291,9 +303,9 @@ class RemoteUI:
                 const.MESSAGE_REMOTE_READY,
             )
 
-        self._certificate_status = CertificateStatus.LOADED
+        self._update_certificate_status(CertificateStatus.INITIAL_LOADED)
         await self._acme.hardening_files()
-        self._certificate_status = CertificateStatus.READY
+        self._update_certificate_status(CertificateStatus.READY)
 
         if self.cloud.client.aiohttp_runner is None:
             _LOGGER.debug("Waiting for aiohttp runner to come available")
@@ -312,7 +324,7 @@ class RemoteUI:
                     const.MESSAGE_LOAD_CERTIFICATE_FAILURE,
                 )
                 await self._recreate_acme(domains, email)
-            self._certificate_status = CertificateStatus.ERROR
+            self._update_certificate_status(CertificateStatus.SSL_CONTEXT_ERROR)
             return False
 
         # Setup snitun / aiohttp wrapper
@@ -384,20 +396,17 @@ class RemoteUI:
         aes_key, aes_iv = generate_aes_keyset()
         try:
             async with async_timeout.timeout(30):
-                resp = await cloud_api.async_remote_token(self.cloud, aes_key, aes_iv)
-                if resp.status == 409:
-                    raise RemoteInsecureVersion
-                if resp.status == 403:
-                    msg = ""
-                    if "application/json" in (resp.content_type or ""):
-                        msg = (await resp.json()).get("message", "")
-                    raise RemoteForbidden(msg)
-                if resp.status not in (200, 201):
-                    raise RemoteBackendError
-        except (TimeoutError, aiohttp.ClientError):
+                data = await self.cloud.instance.snitun_token(
+                    aes_key=aes_key, aes_iv=aes_iv
+                )
+        except TimeoutError:
+            raise RemoteBackendError from None
+        except InstanceApiError as err:
+            if err.status == 409:
+                raise RemoteInsecureVersion from err
+            if err.status == 403:
+                raise RemoteForbidden(err.reason or err) from err
             raise RemoteBackendError from None
-
-        data = await resp.json()
         self._token = SniTunToken(
             data["token"].encode(),
             aes_key,
@@ -528,16 +537,19 @@ class RemoteUI:
 
                 # Renew certificate?
                 if not await self._should_renew_certificates():
+                    self._update_certificate_status(CertificateStatus.READY)
                     continue
 
                 # Renew certificate
                 try:
                     _LOGGER.debug("Renewing certificate")
-                    self._certificate_status = CertificateStatus.GENERATING
+                    self._update_certificate_status(
+                        CertificateStatus.RENEWAL_GENERATING
+                    )
                     await self._acme.issue_certificate()
-                    self._certificate_status = CertificateStatus.LOADED
+                    self._update_certificate_status(CertificateStatus.RENEWAL_LOADED)
                     await self._recreate_backend()
-                    self._certificate_status = CertificateStatus.READY
+                    self._update_certificate_status(CertificateStatus.READY)
                 except AcmeClientError as err:
                     # Only log as warning if we have a certain amount of days left
                     if self._acme.expire_date is None or (
@@ -546,10 +558,10 @@ class RemoteUI:
                         < (utils.utcnow() + timedelta(days=WARN_RENEW_FAILED_DAYS))
                     ):
                         meth = _LOGGER.warning
-                        self._certificate_status = CertificateStatus.ERROR
                     else:
                         meth = _LOGGER.debug
-                        self._certificate_status = CertificateStatus.READY
+
+                    self._update_certificate_status(CertificateStatus.RENEWAL_FAILED)
 
                     meth(
                         "Renewal of ACME certificate failed. Trying again later: %s",
@@ -569,7 +581,7 @@ class RemoteUI:
     async def _check_cname(self, hostname: str) -> list[str]:
         """Get CNAME records for hostname."""
         try:
-            return await cloud_api.async_resolve_cname(self.cloud, hostname)
+            return await self.cloud.instance.resolve_dns_cname(hostname=hostname)
         except (TimeoutError, aiohttp.ClientError):
             _LOGGER.error("Can't resolve CNAME for %s", hostname)
         return []
@@ -591,23 +603,28 @@ class RemoteUI:
 
     async def _should_renew_certificates(self) -> bool:
         """Check if certificates should be renewed."""
-        bad_alias = []
+        self._update_certificate_status(CertificateStatus.VALIDATING)
 
         if TYPE_CHECKING:
             assert self._acme is not None
             assert self.instance_domain is not None
 
         if not self._acme.certificate_available:
+            self._update_certificate_status(CertificateStatus.NO_CERTIFICATE)
             return True
 
-        if self._acme.expire_date is None:
+        utcnow = utils.utcnow()
+
+        if self._acme.expire_date is None or self._acme.expire_date <= utcnow:
+            self._update_certificate_status(CertificateStatus.EXPIRED)
             return True
 
-        if self._acme.expire_date > (
-            utils.utcnow() + timedelta(days=RENEW_IF_EXPIRES_DAYS)
-        ):
+        # Check if certificate is expiring soon
+        if self._acme.expire_date > (utcnow + timedelta(days=RENEW_IF_EXPIRES_DAYS)):
             return False
 
+        self._update_certificate_status(CertificateStatus.EXPIRING_SOON)
+
         check_alias = [
             domain for domain in self._acme.domains if domain != self.instance_domain
         ]
@@ -616,8 +633,8 @@ class RemoteUI:
             return True
 
         # Check if defined alias is still valid:
+        bad_alias = []
         for alias in check_alias:
-            # Check primary entry
             if not await self._custom_domain_dns_configuration_is_valid(
                 self.instance_domain,
                 alias,
@@ -628,6 +645,9 @@ class RemoteUI:
             # No bad configuration detected
             return True
 
+        # Domain validation failed for some aliases
+        self._update_certificate_status(CertificateStatus.DOMAIN_VALIDATION_FAILED)
+
         if self._acme.expire_date > (
             utils.utcnow() + timedelta(days=WARN_RENEW_FAILED_DAYS)
         ):
