@@ -3,23 +3,27 @@
 import asyncio
 from datetime import timedelta
 from ssl import SSLError
-from unittest.mock import patch
+from typing import Any
+from unittest.mock import Mock, patch
 
 from acme import client, messages
+from aiohttp import ClientError
 import pytest
 
 from hass_nabucasa import utils
 from hass_nabucasa.acme import AcmeHandler
 from hass_nabucasa.const import (
+    DISPATCH_CERTIFICATE_STATUS,
     DISPATCH_REMOTE_BACKEND_DOWN,
     DISPATCH_REMOTE_BACKEND_UP,
     DISPATCH_REMOTE_CONNECT,
     DISPATCH_REMOTE_DISCONNECT,
+    CertificateStatus,
 )
+from hass_nabucasa.instance_api import InstanceApiError
 from hass_nabucasa.remote import (
     RENEW_IF_EXPIRES_DAYS,
     WARN_RENEW_FAILED_DAYS,
-    CertificateStatus,
     RemoteUI,
     SubscriptionExpired,
 )
@@ -30,6 +34,26 @@ from .common import MockAcme, MockSnitun
 # pylint: disable=protected-access
 
 
+@pytest.fixture
+def mock_timing():
+    """Mock timing functions for remote tests."""
+    # Store original sleep function to avoid recursion
+    original_sleep = asyncio.sleep
+
+    async def mock_sleep(seconds):
+        """Mock sleep that only sleeps for very short time."""
+        await original_sleep(0.01)  # Always sleep very short time
+
+    with (
+        # Mock timing functions only - be less aggressive to avoid interference
+        patch("hass_nabucasa.utils.next_midnight", return_value=0),
+        patch("random.randint", return_value=0),
+        patch("hass_nabucasa.remote.random.randint", return_value=0),
+        patch("hass_nabucasa.remote.asyncio.sleep", side_effect=mock_sleep),
+    ):
+        yield
+
+
 @pytest.fixture(autouse=True)
 def ignore_context():
     """Ignore ssl context."""
@@ -77,10 +101,24 @@ async def test_load_backend_exists_cert(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Initialize backend."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -135,8 +173,23 @@ async def test_load_backend_exists_cert(
     assert remote._acme_task
     assert remote._reconnect_task
 
-    assert auth_cloud_mock.client.mock_dispatcher[0][0] == DISPATCH_REMOTE_BACKEND_UP
-    assert auth_cloud_mock.client.mock_dispatcher[1][0] == DISPATCH_REMOTE_CONNECT
+    # Check that certificate status updates were dispatched
+    certificate_dispatches = [
+        call
+        for call in auth_cloud_mock.client.mock_dispatcher
+        if call[0] == DISPATCH_CERTIFICATE_STATUS
+    ]
+    # Should have certificate status updates
+    assert len(certificate_dispatches) >= 1
+
+    # Check that backend and connection dispatches happened
+    backend_dispatches = [
+        call
+        for call in auth_cloud_mock.client.mock_dispatcher
+        if call[0] in (DISPATCH_REMOTE_BACKEND_UP, DISPATCH_REMOTE_CONNECT)
+    ]
+    assert any(call[0] == DISPATCH_REMOTE_BACKEND_UP for call in backend_dispatches)
+    assert any(call[0] == DISPATCH_REMOTE_CONNECT for call in backend_dispatches)
 
     await remote.stop()
     await asyncio.sleep(0.1)
@@ -155,6 +208,19 @@ async def test_load_backend_not_exists_cert(
     """Initialize backend."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -213,10 +279,24 @@ async def test_load_and_unload_backend(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Initialize backend."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -276,10 +356,25 @@ async def test_load_backend_exists_wrong_cert(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Initialize backend."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+        "alias": ["example.com"],
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -301,10 +396,10 @@ async def test_load_backend_exists_wrong_cert(
         },
     )
 
-    aioclient_mock.post(
-        "https://example.com/instance/resolve_dns_cname",
-        json=["test.dui.nabu.casa", "_acme-challenge.test.dui.nabu.casa"],
-    )
+    auth_cloud_mock.instance.resolve_dns_cname.return_value = [
+        "test.dui.nabu.casa",
+        "_acme-challenge.test.dui.nabu.casa",
+    ]
 
     auth_cloud_mock.accounts_server = "example.com"
 
@@ -344,10 +439,24 @@ async def test_call_disconnect(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Initialize backend."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -386,10 +495,24 @@ async def test_load_backend_no_autostart(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Initialize backend."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -440,10 +563,24 @@ async def test_get_certificate_details(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Initialize backend."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     assert remote.certificate is None
@@ -489,10 +626,24 @@ async def test_certificate_task_no_backend(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Initialize backend."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -538,10 +689,24 @@ async def test_certificate_task_renew_cert(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Initialize backend."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -582,7 +747,7 @@ async def test_certificate_task_renew_cert(
         assert acme_task.done()
 
 
-async def test_refresh_token_no_sub(auth_cloud_mock):
+async def test_refresh_token_no_sub(auth_cloud_mock, mock_timing):
     """Test that we rais SubscriptionExpired if expired sub."""
     auth_cloud_mock.subscription_expired = True
 
@@ -596,10 +761,21 @@ async def test_load_connect_insecure(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Initialize backend."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+    auth_cloud_mock.instance.snitun_token.side_effect = InstanceApiError(
+        "Remote connection is disabled", status=409
+    )
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -643,9 +819,20 @@ async def test_load_connect_forbidden(
     aioclient_mock,
     snitun_mock,
     caplog,
+    mock_timing,
 ):
     """Initialize backend."""
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+    auth_cloud_mock.instance.snitun_token.side_effect = InstanceApiError(
+        "Remote connection is not allowed", status=403, reason="lorem_ipsum"
+    )
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -684,10 +871,24 @@ async def test_call_disconnect_clean_token(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Initialize backend."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -727,10 +928,25 @@ async def test_recreating_old_certificate_with_bad_dns_config(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Test recreating old certificate with bad DNS config for alias."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+        "alias": ["example.com"],
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -811,10 +1027,25 @@ async def test_warn_about_bad_dns_config_for_old_certificate(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Test warn about old certificate with bad DNS config for alias."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+        "alias": ["example.com"],
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -835,9 +1066,8 @@ async def test_warn_about_bad_dns_config_for_old_certificate(
             "throttling": 400,
         },
     )
-    aioclient_mock.post(
-        "https://example.com/instance/resolve_dns_cname",
-        status=400,
+    auth_cloud_mock.instance.resolve_dns_cname.side_effect = ClientError(
+        "DNS resolution failed"
     )
 
     auth_cloud_mock.accounts_server = "example.com"
@@ -886,10 +1116,25 @@ async def test_regeneration_without_warning_for_good_dns_config(
     mock_cognito,
     aioclient_mock,
     snitun_mock,
+    mock_timing,
 ):
     """Test no warning for good dns config."""
     valid = utcnow() + timedelta(days=1)
     auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+        "alias": ["example.com"],
+    }
+    auth_cloud_mock.instance.snitun_token.return_value = {
+        "token": "test-token",
+        "server": "rest-remote.nabu.casa",
+        "valid": valid.timestamp(),
+        "throttling": 400,
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -910,10 +1155,10 @@ async def test_regeneration_without_warning_for_good_dns_config(
             "throttling": 400,
         },
     )
-    aioclient_mock.post(
-        "https://example.com/instance/resolve_dns_cname",
-        json=["test.dui.nabu.casa", "_acme-challenge.test.dui.nabu.casa"],
-    )
+    auth_cloud_mock.instance.resolve_dns_cname.return_value = [
+        "test.dui.nabu.casa",
+        "_acme-challenge.test.dui.nabu.casa",
+    ]
 
     auth_cloud_mock.accounts_server = "example.com"
 
@@ -955,6 +1200,20 @@ async def test_regeneration_without_warning_for_good_dns_config(
             },
             True,
         ),
+        (
+            {
+                "type": "urn:ietf:params:acme:error:malformed",
+                "detail": "Unable to validate JWS",
+            },
+            True,
+        ),
+        (
+            {
+                "type": "urn:ietf:params:acme:error:malformed",
+                "detail": "Unable to validate JWS :: Invalid Content-Type header on",
+            },
+            True,
+        ),
         (
             {
                 "type": "urn:ietf:params:acme:error:malformed",
@@ -978,6 +1237,7 @@ async def test_acme_client_new_order_errors(
     snitun_mock,
     json_error,
     should_reset,
+    mock_timing,
 ):
     """Initialize backend."""
     auth_cloud_mock.servicehandlers_server = "test.local"
@@ -1010,6 +1270,12 @@ async def test_acme_client_new_order_errors(
         async def reset_acme(self):
             self.call_reset = True
 
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -1023,14 +1289,127 @@ async def test_acme_client_new_order_errors(
 
     with patch(
         "hass_nabucasa.remote.AcmeHandler",
-        return_value=_MockAcme(auth_cloud_mock, [], "test@nabucasa.inc"),
+        return_value=_MockAcme(auth_cloud_mock, [], "test@nabucasa.inc", Mock()),
     ):
         assert remote._certificate_status is None
         await remote.load_backend()
 
     await asyncio.sleep(0.1)
     assert remote._acme.call_reset == should_reset
-    assert remote._certificate_status is CertificateStatus.ERROR
+    assert remote._certificate_status is CertificateStatus.INITIAL_CERT_ERROR
+
+    await remote.stop()
+
+
+@pytest.mark.parametrize(
+    ("json_error", "should_reset"),
+    (
+        (
+            {
+                "type": "urn:ietf:params:acme:error:malformed",
+                "detail": "JWS verification error",
+            },
+            True,
+        ),
+        (
+            {
+                "type": "urn:ietf:params:acme:error:malformed",
+                "detail": "Unable to validate JWS",
+            },
+            True,
+        ),
+        (
+            {
+                "type": "urn:ietf:params:acme:error:malformed",
+                "detail": "Unable to validate JWS :: Invalid Content-Type header on",
+            },
+            True,
+        ),
+        (
+            {
+                "type": "urn:ietf:params:acme:error:malformed",
+                "detail": "Some other malformed reason",
+            },
+            False,
+        ),
+        (
+            {
+                "type": "about:blank",
+                "detail": "Boom",
+            },
+            False,
+        ),
+    ),
+)
+async def test_acme_client_create_client_jws_errors(
+    auth_cloud_mock,
+    mock_cognito,
+    aioclient_mock,
+    snitun_mock,
+    json_error,
+    should_reset,
+    mock_timing,
+):
+    """Test ACME client creation JWS error handling."""
+    auth_cloud_mock.servicehandlers_server = "test.local"
+
+    class _MockAcmeClient(client.ClientV2):
+        def __init__(self) -> None:
+            pass
+
+        @classmethod
+        def get_directory(cls, *_: Any, **_kwargs: Any) -> None:
+            raise messages.Error.from_json(json_error)
+
+    class _MockAcme(AcmeHandler):
+        call_reset = False
+        cloud = auth_cloud_mock
+
+        @property
+        def certificate_available(self):
+            return True
+
+        @property
+        def alternative_names(self):
+            return ["test.dui.nabu.casa"]
+
+        def _load_account_key(self):
+            self._account_jwk = Mock()
+
+        def _create_client(self):
+            with patch("hass_nabucasa.acme.client.ClientV2", _MockAcmeClient):
+                super()._create_client()
+
+        async def reset_acme(self):
+            self.call_reset = True
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+
+    remote = RemoteUI(auth_cloud_mock)
+
+    aioclient_mock.post(
+        "https://test.local/instance/register",
+        json={
+            "domain": "test.dui.nabu.casa",
+            "email": "test@nabucasa.inc",
+            "server": "rest-remote.nabu.casa",
+        },
+    )
+
+    with patch(
+        "hass_nabucasa.remote.AcmeHandler",
+        return_value=_MockAcme(auth_cloud_mock, [], "test@nabucasa.inc", Mock()),
+    ):
+        assert remote._certificate_status is None
+        await remote.load_backend()
+
+    await asyncio.sleep(0.1)
+    assert remote._acme.call_reset == should_reset
+    assert remote._certificate_status is CertificateStatus.INITIAL_CERT_ERROR
 
     await remote.stop()
 
@@ -1056,10 +1435,17 @@ async def test_context_error_handling(
     snitun_mock,
     reason,
     should_reset,
+    mock_timing,
 ):
     """Test that we reset if we hit an error reason that require resetting."""
     auth_cloud_mock.servicehandlers_server = "test.local"
 
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+    }
+
     remote = RemoteUI(auth_cloud_mock)
 
     aioclient_mock.post(
@@ -1083,6 +1469,152 @@ async def test_context_error_handling(
 
     await asyncio.sleep(0.1)
     assert remote._acme.call_reset == should_reset
-    assert remote._certificate_status is CertificateStatus.ERROR
+    assert remote._certificate_status is CertificateStatus.SSL_CONTEXT_ERROR
 
     await remote.stop()
+
+
+async def test_certificate_status_dispatcher(auth_cloud_mock):
+    """Test certificate status dispatcher functionality."""
+    remote = RemoteUI(auth_cloud_mock)
+
+    # Test status update dispatching
+    remote._update_certificate_status(CertificateStatus.LOADING)
+
+    # Check that dispatcher was called
+    assert len(auth_cloud_mock.client.mock_dispatcher) == 1
+    assert auth_cloud_mock.client.mock_dispatcher[0] == (
+        DISPATCH_CERTIFICATE_STATUS,
+        CertificateStatus.LOADING,
+    )
+
+    # Test second status update
+    remote._update_certificate_status(CertificateStatus.ERROR)
+
+    assert len(auth_cloud_mock.client.mock_dispatcher) == 2
+    assert auth_cloud_mock.client.mock_dispatcher[1] == (
+        DISPATCH_CERTIFICATE_STATUS,
+        CertificateStatus.ERROR,
+    )
+
+    # Test duplicate status update (should be ignored)
+    remote._update_certificate_status(CertificateStatus.ERROR)
+
+    # Should still be 2 calls since status didn't change
+    assert len(auth_cloud_mock.client.mock_dispatcher) == 2
+
+
+async def test_recreate_acme_calls_reset_when_acme_exists(auth_cloud_mock):
+    """Test that _recreate_acme calls reset_acme when _acme exists."""
+    remote = RemoteUI(auth_cloud_mock)
+
+    mock_acme = MockAcme()
+    remote._acme = mock_acme
+
+    await remote._recreate_acme(["new.example.com"], "new@example.com")
+
+    assert mock_acme.call_reset is True
+    assert remote._acme is not mock_acme
+
+
+async def test_recreate_acme_with_certificate_available(auth_cloud_mock):
+    """Test _recreate_acme behavior when certificate is available."""
+    remote = RemoteUI(auth_cloud_mock)
+
+    mock_acme = MockAcme()
+    remote._acme = mock_acme
+
+    await remote._recreate_acme(["new.example.com"], "new@example.com")
+
+    assert mock_acme.call_reset is True
+    assert remote._acme is not mock_acme
+
+
+async def test_recreate_acme_without_certificate_available(auth_cloud_mock):
+    """Test _recreate_acme behavior when certificate is not available."""
+    remote = RemoteUI(auth_cloud_mock)
+
+    mock_acme = MockAcme()
+    mock_acme.common_name = None
+    remote._acme = mock_acme
+
+    await remote._recreate_acme(["new.example.com"], "new@example.com")
+
+    assert mock_acme.call_reset is True
+    assert remote._acme is not mock_acme
+
+
+async def test_recreate_acme_with_error_status(auth_cloud_mock):
+    """Test _recreate_acme behavior when certificate status is ERROR."""
+    remote = RemoteUI(auth_cloud_mock)
+
+    mock_acme = MockAcme()
+    remote._acme = mock_acme
+    remote._certificate_status = CertificateStatus.ERROR
+
+    await remote._recreate_acme(["new.example.com"], "new@example.com")
+
+    assert mock_acme.call_reset is True
+    assert remote._acme is not mock_acme
+
+
+async def test_recreate_acme_when_no_acme_exists(auth_cloud_mock):
+    """Test _recreate_acme behavior when no ACME handler exists."""
+    remote = RemoteUI(auth_cloud_mock)
+
+    remote._acme = None
+
+    await remote._recreate_acme(["test.example.com"], "test@example.com")
+
+    assert remote._acme is not None
+
+
+async def test_recreate_acme_integration_during_load_backend(
+    auth_cloud_mock,
+    valid_acme_mock,
+    mock_cognito,
+    aioclient_mock,
+    snitun_mock,
+):
+    """Test _recreate_acme integration during load_backend with domain changes."""
+    auth_cloud_mock.servicehandlers_server = "test.local"
+
+    auth_cloud_mock.instance.register.return_value = {
+        "domain": "test.dui.nabu.casa",
+        "email": "test@nabucasa.inc",
+        "server": "rest-remote.nabu.casa",
+        "alias": ["new-alias.com"],
+    }
+
+    remote = RemoteUI(auth_cloud_mock)
+
+    aioclient_mock.post(
+        "https://test.local/instance/register",
+        json={
+            "domain": "test.dui.nabu.casa",
+            "email": "test@nabucasa.inc",
+            "server": "rest-remote.nabu.casa",
+            "alias": ["new-alias.com"],
+        },
+    )
+
+    valid_acme_mock.common_name = "test.dui.nabu.casa"
+    valid_acme_mock.alternative_names = ["test.dui.nabu.casa", "old-alias.com"]
+
+    auth_cloud_mock.instance.resolve_dns_cname.return_value = [
+        "test.dui.nabu.casa",
+        "_acme-challenge.test.dui.nabu.casa",
+    ]
+    auth_cloud_mock.accounts_server = "example.com"
+
+    await remote.load_backend()
+    await asyncio.sleep(0.1)
+
+    assert valid_acme_mock.call_reset is True
+
+    expected_domains = ["test.dui.nabu.casa", "new-alias.com"]
+    assert valid_acme_mock.init_args == (
+        auth_cloud_mock,
+        expected_domains,
+        "test@nabucasa.inc",
+    )
